lmem(1)=6;mem(1)="days"
lmem(2)=6;mem(2)="years"
lmem(3)=13;mem(3)="solar masses"
lmem(4)=3;mem(4)="AU"
lmem(5)=3;mem(5)="no"
lmem(6)=3;mem(6)="yes"
lmem(7)=3;mem(7)="low"
lmem(8)=6;mem(8)="medium"
lmem(9)=4;mem(9)="high"
lmem(10)=0;mem(10)=""
lmem(11)=33;mem(11)="Integration parameters"
lmem(12)=33;mem(12)="----------------------"
lmem(13)=14;mem(13)="Algorithm:"
lmem(14)=38;mem(14)="Second-order mixed-variable symplectic"
lmem(15)=24;mem(15)="Bulirsch-Stoer (general)"
lmem(16)=37;mem(16)="Bulirsch-Stoer (conservative systems)"
lmem(17)=16;mem(17)="15th-order RADAU"
lmem(18)=0;mem(18)=""
lmem(19)=0;mem(19)=""
lmem(20)=0;mem(20)=""
lmem(21)=0;mem(21)=""
lmem(22)=5;mem(22)="Test"
lmem(23)=48;mem(23)="Hybrid symplectic integrator (mixed coordinates)"
lmem(24)=44;mem(24)="Hybrid symplectic (close binary coordinates)"
lmem(25)=43;mem(25)="Hybrid symplectic (wide binary coordinates)"
lmem(26)=32;mem(26)="Integration start epoch:"
lmem(27)=32;mem(27)="Integration stop epoch:"
lmem(28)=32;mem(28)="Output interval:"
lmem(29)=32;mem(29)="Element origin:"
lmem(30)=31;mem(30)="Initial timestep:"
lmem(31)=36;mem(31)="Accuracy parameter:"
lmem(32)=36;mem(32)="Central mass:"
lmem(33)=36;mem(33)="J_2:"
lmem(34)=36;mem(34)="J_4:"
lmem(35)=36;mem(35)="J_6:"
lmem(36)=36;mem(36)="Ejection distance:"
lmem(37)=36;mem(37)="Radius of central body:"
lmem(38)=29;mem(38)="Number of Big bodies:"
lmem(39)=29;mem(39)="Number of Small bodies:"
lmem(40)=37;mem(40)="Output precision:"
lmem(41)=40;mem(41)="Includes collisions:"
lmem(42)=40;mem(42)="Includes fragmentation:"
lmem(43)=0;mem(43)=""
lmem(44)=0;mem(44)=""
lmem(45)=40;mem(45)="Includes relativity:"
lmem(46)=40;mem(46)="Includes user-defined force routine:"
lmem(47)=10;mem(47)="barycentre"
lmem(48)=12;mem(48)="central body"
lmem(49)=0;mem(49)=""
lmem(50)=0;mem(50)=""
lmem(51)=30;mem(51)="Integration details"
lmem(52)=30;mem(52)="-------------------"
lmem(53)=29;mem(53)="Initial energy:"
lmem(54)=29;mem(54)="Initial angular momentum:"
lmem(55)=65;mem(55)="Integrating massive bodies and particles up to the same epoch."
lmem(56)=34;mem(56)="Beginning the main integration."
lmem(57)=24;mem(57)="Integration complete."
lmem(58)=48;mem(58)="Fractional energy change due to integrator:"
lmem(59)=48;mem(59)="Fractional angular momentum change:"
lmem(60)=57;mem(60)="Fractional energy change due to collisions/ejections:"
lmem(61)=57;mem(61)="Fractional angular momentum change:"
lmem(62)=47;mem(62)="Continuing integration from dump files at"
lmem(63)=6;mem(63)="Time:"
lmem(64)=6;mem(64)="Date:"
lmem(65)=9;mem(65)="dE/E:"
lmem(66)=9;mem(66)="dL/L:"
lmem(67)=35;mem(67)="collided with the central body at"
lmem(68)=12;mem(68)="ejected at"
lmem(69)=12;mem(69)="was hit by"
lmem(70)=34;mem(70)="removed due to an encounter with"
lmem(71)=4;mem(71)="at"
lmem(72)=26;mem(72)="solar masses AU^2 day^-2"
lmem(73)=26;mem(73)="solar masses AU^2 day^-1"
lmem(74)=36;mem(74)="lost mass due to rotational breakup"
lmem(75)=24;mem(75)="removed due to small a"
lmem(76)=0;mem(76)=""
lmem(77)=0;mem(77)=""
lmem(78)=0;mem(78)=""
lmem(79)=0;mem(79)=""
lmem(80)=0;mem(80)=""
lmem(81)=8;mem(81)="ERROR:"
lmem(82)=49;mem(82)="Modify mercury.inc and recompile Mercury."
lmem(83)=62;mem(83)="Check the file containing initial data for Big bodies."
lmem(84)=64;mem(84)="Check the file containing initial data for Small bodies."
lmem(85)=57;mem(85)="Check the file containing integration parameters."
lmem(86)=22;mem(86)="Check files.in"
lmem(87)=27;mem(87)="This file already exists:"
lmem(88)=34;mem(88)="This file is needed to continue:"
lmem(89)=30;mem(89)="This filename is duplicated:"
lmem(90)=40;mem(90)="The total number of bodies exceeds NMAX."
lmem(91)=68;mem(91)="Data style on first line must be Cartesian, Asteroidal or Cometary"
lmem(92)=68;mem(92)="You cannot integrate non-gravitational forces using this algorithm."
lmem(93)=64;mem(93)="You cannot integrate a user-defined force using this algorithm."
lmem(94)=64;mem(94)="You cannot integrate massive Small bodies using this algorithm."
lmem(95)=66;mem(95)="Massive Small bodies must have the same epoch as the Big bodies."
lmem(96)=49;mem(96)="Check character implies input file is corrupted."
lmem(97)=62;mem(97)="Mass, density, encounter limit must be >= 0 for this object:"
lmem(98)=46;mem(98)="This integration algorithm is not available:"
lmem(99)=50;mem(99)="A problem occurred reading the parameter on line"
lmem(100)=50;mem(100)="A problem occurred reading data for this object:"
lmem(101)=56;mem(101)="A problem occured reading the epoch for the Big bodies."
lmem(102)=67;mem(102)="You cannot use non-zero J2,J4,J6 using the close-binary algorithm."
lmem(103)=34;mem(103)="Two objects both have this name:"
lmem(104)=36;mem(104)="is corrupted at line number:"
lmem(105)=42;mem(105)="Central-body radius exceeds maximum radius."
lmem(106)=68;mem(106)="Maximum/Central radius is large. Output precision will be degraded."
lmem(107)=58;mem(107)="Coordinate origin must be Central, Barycentric or Jacobi."
lmem(108)=0;mem(108)=""
lmem(109)=0;mem(109)=""
lmem(110)=0;mem(110)=""
lmem(111)=0;mem(111)=""
lmem(112)=0;mem(112)=""
lmem(113)=0;mem(113)=""
lmem(114)=0;mem(114)=""
lmem(115)=0;mem(115)=""
lmem(116)=0;mem(116)=""
lmem(117)=0;mem(117)=""
lmem(118)=0;mem(118)=""
lmem(119)=0;mem(119)=""
lmem(120)=0;mem(120)=""
lmem(121)=10;mem(121)="WARNING:"
lmem(122)=53;mem(122)="Truncating the name of this object to 8 characters:"
lmem(123)=30;mem(123)="Main integration is backwards."
lmem(124)=26;mem(124)="No Big bodies are present."
lmem(125)=28;mem(125)="No Small bodies are present."
lmem(126)=50;mem(126)="Stopping integration due to an encounter between"
lmem(127)=45;mem(127)="Throwing this object into the central body:"
lmem(128)=42;mem(128)="Setting output threshhold DA to infinity."
lmem(129)=42;mem(129)="Setting output threshhold DE to infinity."
lmem(130)=42;mem(130)="Setting output threshhold DI to infinity."
lmem(131)=43;mem(131)="Increasing the radius of the central body."
lmem(132)=56;mem(132)="Total number of current close encounters exceeds CMAX."
lmem(133)=0;mem(133)=""
lmem(134)=0;mem(134)=""
lmem(135)=0;mem(135)=""
lmem(136)=0;mem(136)=""
lmem(137)=0;mem(137)=""
lmem(138)=0;mem(138)=""
lmem(139)=0;mem(139)=""
lmem(140)=0;mem(140)=""
lmem(141)=0;mem(141)=""
lmem(142)=0;mem(142)=""
lmem(143)=0;mem(143)=""
lmem(144)=0;mem(144)=""
lmem(145)=0;mem(145)=""
lmem(146)=0;mem(146)=""
lmem(147)=0;mem(147)=""
lmem(148)=0;mem(148)=""
lmem(149)=0;mem(149)=""
lmem(150)=0;mem(150)=""
lmem(151)=67;mem(151)=")O+_05 Integration parameters (WARNING: Do not delete this line!!)"
lmem(152)=66;mem(152)=")O+_05 Big-body initial data (WARNING: Do not delete this line!!)"
lmem(153)=68;mem(153)=")O+_05 Small-body initial data (WARNING: Do not delete this line!!)"
lmem(154)=39;mem(154)=") Lines beginning with `)' are ignored."
lmem(155)=70;mem(155)=")---------------------------------------------------------------------"
lmem(156)=43;mem(156)="style (Cartesian, Asteroidal, Cometary) ="
lmem(157)=20;mem(157)="epoch (in days) ="
lmem(158)=35;mem(158)=") Important integration parameters:"
lmem(159)=48;mem(159)="algorithm (MVS, BS, BS2, RADAU, HYBRID etc) ="
lmem(160)=21;mem(160)="start time (days) ="
lmem(161)=20;mem(161)="stop time (days) ="
lmem(162)=26;mem(162)="output interval (days) ="
lmem(163)=19;mem(163)="timestep (days) ="
lmem(164)=22;mem(164)="accuracy parameter ="
lmem(165)=22;mem(165)=") Integration options:"
lmem(166)=44;mem(166)="stop integration after a close encounter ="
lmem(167)=29;mem(167)="allow collisions to occur ="
lmem(168)=37;mem(168)="include collisional fragmentation ="
lmem(169)=33;mem(169)="express time in days or years ="
lmem(170)=51;mem(170)="express time relative to integration start time ="
lmem(171)=20;mem(171)="output precision ="
lmem(172)=24;mem(172)="< Not used at present >"
lmem(173)=37;mem(173)="include relativity in integration ="
lmem(174)=30;mem(174)="include user-defined force ="
lmem(175)=52;mem(175)=") These parameters do not need to be adjusted often:"
lmem(176)=26;mem(176)="ejection distance (AU) ="
lmem(177)=31;mem(177)="radius of central body (AU) ="
lmem(178)=31;mem(178)="central mass (solar masses) ="
lmem(179)=14;mem(179)="central J2 ="
lmem(180)=14;mem(180)="central J4 ="
lmem(181)=14;mem(181)="central J6 ="
lmem(182)=24;mem(182)="< Not used at present >"
lmem(183)=24;mem(183)="< Not used at present >"
lmem(184)=45;mem(184)="Hybrid integrator changeover (Hill radii) ="
lmem(185)=42;mem(185)="number of timesteps between data dumps ="
lmem(186)=48;mem(186)="number of timesteps between periodic effects ="
lmem(187)=41;mem(187)="origin (Central, Barycentric, Jacobi) ="
lmem(188)=0;mem(188)=""
lmem(189)=0;mem(189)=""
lmem(190)=0;mem(190)=""
lmem(191)=0;mem(191)=""
lmem(192)=0;mem(192)=""
lmem(193)=0;mem(193)=""
lmem(194)=0;mem(194)=""
lmem(195)=0;mem(195)=""
lmem(196)=0;mem(196)=""
lmem(197)=0;mem(197)=""
lmem(198)=0;mem(198)=""
lmem(199)=0;mem(199)=""
lmem(200)=0;mem(200)=""
